module Slack
	module RPC

		# ## Slack::RPC::Command
		# A class that represents Slack-RPC style command
		#
		class Command

			# ### Slack::RPC::Commnad.new(conn, command, sub_commands)
			# Creates a new instance of `Slack::RPC::Command` class
			#
			def initialize(connection, command, sub_commands)
				@connection   = connection
				@command      = command
				@sub_commands = sub_commands
			end

			# ### Slack::RPC::Command.invoke_command
			# Invokes Slack RPC-Style command
			#
			def call(sub_command, params, &block)
				@connection.call(@command, sub_command, params, &block)
			end
			
			# ### Slack::RPC::Command.respond_to?(name)
			# Returns true if obj responds to the given method.
			#
			def respond_to?(name)
				@sub_commands.include?(name) || super
			end

			# ### Slack::RPC::Command.method_missing(name, *args, &block)
			# Invoked by Ruby when obj is sent a message it cannot handle.
			#
			def method_missing(name, *args, &block)
				if @sub_commands.include?(name.to_s) then
					params = (args.length == 1 && args[0].class == Hash) ? args[0] : {}
					call(name.to_s, params, &block)
				else
					super
				end
			end

		end

	end
end
