require "faraday"
require "faraday_middleware"
require "slack/rpc/response"

module Slack
	module RPC

		# ## Slack::RPC::Connection
		# A class for invoke Slack RPC-Style command
		#
		class Connection

			attr_accessor :token

			# ### Slack::RPC::Connection.BASE_URL
			# A base url to invoke Slack RPC-Style command
			#
			BASE_URL = "https://slack.com/api"

			# ### Slack::RPC::Connection.HEADERS
			# HTTP request headers as Hash object
			#
			HEADERS = {
				"Accept" => "application/json",
				"User-Agent" => "Slack Ruby Gem #{Slack::VERSION}"
			}

			# ### Slack::RPC::Connection.new(token)
			# Creates a new instance of `Slack::RPC::Connection` class
			#
			def initialize(token = nil)
				@token = token
			end

			# ### Slack::RPC::Connection.call(command, sub_commnad, params, &block)
			# Call Slack RPC-Style command
			#
			def call(command, sub_command, params, &block)
				params = params.clone.merge({:token => @token}) if @token
				faraday_response = connection.get("#{command}.#{sub_command}", params)
				response = Response.new(faraday_response)
				if block then
					yield response
				else
					response
				end
			end

		private

			# ### Slack::RPC::Connection.connection
			# retrieve Faraday Connection object
			#
			def connection
				@connection ||= Faraday.new(:headers => HEADERS, :url => BASE_URL) { |faraday|
					faraday.request  :url_encoded
					faraday.response :json
					faraday.adapter  Faraday.default_adapter
				}
			end

		end
	end
end

